// (c) Slavomir Tuleja, 5. november 2002
// JApplet sluzi na simulaciu orbitalneho pohybu druzice okolo Zeme

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.event.*;


public class JOrbitLab extends JApplet
   implements Runnable, ActionListener, MouseMotionListener, MouseListener{

   public BufferedImage image;
   //...komponenta - obrazok
   private ImageIcon iconPlay,iconStop,iconStepForward, iconStepBack;
   private MediaTracker tracker;
   private Image imMap,imPlay,imStop,imStepForward,imStepBack;

   private Graphics2D g2;
   //...objekt, ktory vie kreslit ciary, kruhy, ...
   private JButton btnStartStop, btnNastav, btnDopredu, btnDozadu, btnStav, btnReset;
   private JPanel pnlButtons,pnlLabels,pnlSouth;
   private JLabel lblTime, lblH, lblSirka, lblDlzka;
   private Platno platno;
   //...nas vlastny objekt sluziaci na zobrazenie image
   private TextualInputWindow windowNastavenia;
   private Container cp;
   //...kontainer - obsah okna appletu
   private Scale sc;
   //...nas objekt pre transformaciu suradnic na pixely a opacne
   private Thread vlakno;
   //...podproces (vlakno) v ktorom pobezi animacia

   private OrbitODE ode=new Orbit();
   //...objekt diferencialnej rovnice kyvadla
   public ODEAdaptiveSolver odeAdaptiveSolver;
   //...objekt riesitela tejto dif. rovnice

   private double t,phi,theta,r;
   private double vR,vTheta,vPhi;
   private double rd,thetad,phid;
   public double phiInitial, thetaInitial, rInitial,vRInitial, vThetaInitial,vPhiInitial,dtInitial;
   public double dt;
   private final double Rz=6378e3;
   private double thetaStare,phiStare;
   private boolean koniec,dopad,cursorOver=false;
   private int w=400,h=200; //sirka a vyska image
   private int ci, cj;//suradnice kurzora nad platnom
   private double thetaS;

   public String parH, parFi, parLam, parVR, parVNorth, parVEast, parDT, parFiSun;

   public void init(){
      //toto je kvoli pocitadlu appletov:
      getImage(getCodeBase(), "obrazokA.php?nazov=JOrbitLab&codeBase=" + getCodeBase());

      //nacitanie parametrov appletu
      parH = getParameter("h");
      parFi = getParameter("fi");
      parLam = getParameter("lam");
      parVR = getParameter("vNahor");
      parVNorth = getParameter("vSever");
      parVEast = getParameter("vVychod");
      parDT = getParameter("dt");
      parFiSun = getParameter("fiSlnko");


      //vytvorenie GUI
      cp=getContentPane();
      image=new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
      platno=new Platno(image,this);
      //nacitanie gifu s mapou Zeme
      tracker = new MediaTracker(this);
      imMap = getImage(getCodeBase(), "earth.gif");
      tracker.addImage(imMap,0);
      imPlay = getImage(getCodeBase(), "Play.gif");
      tracker.addImage(imPlay,1);
      imStop = getImage(getCodeBase(), "Stop.gif");
      tracker.addImage(imStop,2);
      imStepForward = getImage(getCodeBase(), "StepForward.gif");
      tracker.addImage(imStepForward,3);
      imStepBack = getImage(getCodeBase(), "StepBack.gif");
      tracker.addImage(imStepBack,4);

      while(!tracker.checkAll(true))
         try
            {
                Thread.sleep(100);
            }
         catch(InterruptedException _ex) {
         }

      iconPlay = new ImageIcon(imPlay);
      iconStop = new ImageIcon(imStop);
      iconStepForward = new ImageIcon(imStepForward);
      iconStepBack = new ImageIcon(imStepBack);

      btnStartStop=new JButton("tart",iconPlay);
      btnNastav=new JButton("Nastavenia");
      btnDopredu = new JButton("Krok",iconStepForward);
      btnDozadu = new JButton("Krok",iconStepBack);
      btnReset = new JButton("Reset");
      btnStav = new JButton("Stav druice");
      lblTime= new JLabel("t = 0 s");


      lblH=new JLabel("h = ".concat(MyUtils.roundOff(Double.valueOf(parH).doubleValue()/1000.0,1)).concat(" km"));
      lblSirka=new JLabel("z..: ");
      lblDlzka=new JLabel("z.d.: ");

      pnlButtons=new JPanel();
      pnlButtons.setLayout(new GridLayout(2,3));
      pnlButtons.add(btnStartStop);
      pnlButtons.add(btnDozadu);
      pnlButtons.add(btnDopredu);
      pnlButtons.add(btnNastav);
      pnlButtons.add(btnStav);
      pnlButtons.add(btnReset);

      pnlLabels = new JPanel();
      pnlLabels.setLayout(new GridLayout(1,4));
      pnlLabels.add(lblTime);
      pnlLabels.add(lblH);
      pnlLabels.add(lblSirka);
      pnlLabels.add(lblDlzka);

      pnlSouth = new JPanel();
      pnlSouth.setLayout(new BoxLayout(pnlSouth,BoxLayout.Y_AXIS));
      pnlSouth.add(pnlLabels);
      pnlSouth.add(Box.createRigidArea(new Dimension(0,5)));
      pnlSouth.add(pnlButtons);

      cp.setLayout(new BorderLayout());
      cp.add(platno,BorderLayout.CENTER);
      cp.add(pnlSouth,BorderLayout.SOUTH);

      btnStartStop.addActionListener(this);
      btnDopredu.addActionListener(new StepForwardHandler(this));
      btnDozadu.addActionListener(new StepBackHandler(this));
      btnNastav.addActionListener(this);
      btnReset.addActionListener(this);
      btnStav.addActionListener(this);

      g2=image.createGraphics();
      sc=new Scale(-Math.PI,Math.PI,Math.PI,0,0,w,0,h);

      platno.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
      platno.addMouseMotionListener(this);
      platno.addMouseListener(this);

      windowNastavenia = new TextualInputWindow(this);
      rInitial=Double.valueOf(parH).doubleValue()+Rz;
      thetaInitial=90-Double.valueOf(parFi).doubleValue();
      phiInitial=toDlzka(Double.valueOf(parLam).doubleValue()/180.0*Math.PI);
      vRInitial=Double.valueOf(parVR).doubleValue();
      vThetaInitial=-Double.valueOf(parVNorth).doubleValue();
      vPhiInitial=Double.valueOf(parVEast).doubleValue();
      dtInitial=Double.valueOf(parDT).doubleValue();
      thetaS=Math.PI/2.0-Double.valueOf(parFiSun).doubleValue()/180.0*Math.PI;

      reset();
      nastavSimulaciu(rInitial,thetaInitial,phiInitial,vRInitial,vThetaInitial,vPhiInitial,0,dtInitial);
   }

   public void reset(){
      nastavSimulaciu(rInitial,thetaInitial,phiInitial,vRInitial,vThetaInitial,vPhiInitial,0,dtInitial);
   }

   public void nastavSimulaciu(double r, double theta, double phi, double vR, double vTheta, double vPhi, double t, double dt) {
      //vstupne uhly su v STUPNOCH!!!
      koniec=false;
      dopad=false;
      this.t=t;
      this.r=r;
      this.theta=theta/180.0*Math.PI;
      this.phi=phi/180.0*Math.PI;
      this.dt=dt;

      //tu sa nastavia zaciatocne podmienky InitialConditions
      ode.setIC(r,theta/180.0*Math.PI,phi/180.0*Math.PI,vR,vTheta,vPhi);

      lblTime.setText("t = ".concat(MyUtils.roundOff(t,0)).concat(" s"));
      lblH.setText("h = ".concat(MyUtils.roundOff((r-Rz)/1000.0,1)).concat(" km"));
      lblSirka.setText("z..: ".concat(MyUtils.roundOff(toSirka(this.theta),1)).concat(" "));
      lblDlzka.setText("z.d.: ".concat(MyUtils.roundOff(toDlzka(this.phi),1)).concat(" "));

      //inicializacia ODESolvera
      odeAdaptiveSolver=new RK45MultiStep(ode);
      odeAdaptiveSolver.initialize(dt);
      odeAdaptiveSolver.setTolerance(1e-6);
      //...tolerancia pri jednom kroku
      odeAdaptiveSolver.setStepSize(dt);
      //...dlzka fixneho kroku

      kresliSpodok(g2);
      platno.repaint();
   }

   public void actionPerformed(ActionEvent e){
      if(e.getActionCommand().equals("Stop")){
         //po stisnuti STOP
         koniec=true;
         btnStartStop.setText("tart");
         btnStartStop.setIcon(iconPlay);
         btnDopredu.setEnabled(true);
         btnDozadu.setEnabled(true);
         btnNastav.setEnabled(true);
         btnReset.setEnabled(true);
         btnStav.setEnabled(true);
      }
      else if(e.getActionCommand().equals("tart")&&(!dopad)){
         //po stisnuti START
         koniec=false;
         btnStartStop.setText("Stop");
         btnStartStop.setIcon(iconStop);
         btnDopredu.setEnabled(false);
         btnDozadu.setEnabled(false);
         btnNastav.setEnabled(false);
         btnReset.setEnabled(false);
         btnStav.setEnabled(false);
         vlakno=new Thread(this);
         vlakno.start();
      }

      else if(e.getActionCommand().equals("Reset")){
         reset();
         repaint();
      }
      else if(e.getActionCommand().equals("Stav druice")){
         vypisStav();
      }
      else if(e.getActionCommand().equals("Nastavenia")){
         //po stisnuti NASTAVENIA
         windowNastavenia.setLocation();
         windowNastavenia.show();
      }

   }

   public void vypisStav(){
      String s;
      s=new String("Stav druice:\n");
      s=s.concat("Vka:  ").concat(MyUtils.roundOff(ode.getR()-Rz,0)).concat(" m\n");
      s=s.concat("Zem. rka:  ").concat(MyUtils.roundOff(toSirka(ode.getTheta()),1)).concat(" \u00b0 \n");
      s=s.concat("Zem. dka:  ").concat(MyUtils.roundOff(toDlzka(ode.getPhi()-7.272e-5*ode.getT()),1)).concat(" \u00b0 \n");
      s=s.concat("Rchlos nahor:  ").concat(MyUtils.roundOff(ode.getVR(),1)).concat(" m/s \n");
      s=s.concat("Rchlos na sever:  ").concat(MyUtils.roundOff(-ode.getVTheta(),1)).concat(" m/s \n");
      s=s.concat("Rchlos na vchod:  ").concat(MyUtils.roundOff(ode.getVPhi(),1)).concat(" m/s \n");
      s=s.concat("Vekos rchlosti:  ").concat(MyUtils.roundOff(Math.sqrt(ode.getVR()*ode.getVR()+ode.getVPhi()*ode.getVPhi()+ode.getVTheta()*ode.getVTheta()),1)).concat(" m/s \n");

      JOptionPane.showMessageDialog(this,s,"Aktulny stav druice",JOptionPane.INFORMATION_MESSAGE);
   }

   public void kresliVrch(Graphics2D g2){
      kresliDruzicu(g2);
      kresliTienASlnko(g2);
      kresliOblastViditelnosti(g2);
      if(cursorOver) kresliRamcek(g2);
   }

   private void kresliRamcek(Graphics2D g2){
      //vykresli ramcek so suradnicami pod kurzorom
      double cPhi=toDlzka(sc.iTox(ci));
      double cTheta=toSirka(sc.jToy(cj));

      g2.setColor(new Color(207,207,255));
      g2.fillRect(0,180,159,19);
      g2.setColor(Color.black);
      g2.drawRect(0,180,159,19);
      g2.drawString("z..: ".concat(MyUtils.roundOff(cTheta,1)).concat(" "),10,195);
      g2.drawString("z.d.: ".concat(MyUtils.roundOff(cPhi,1)).concat(" "),90,195);
   }

   private void kresliDruzicu(Graphics2D g2){
      if(dopad){
         g2.setColor(Color.red);
      }
      else{
         g2.setColor(Color.black);
      }
      g2.fillOval(sc.xToi(adjust(phi))-2,sc.yToj(theta)-2,4,4);

      //ramik okolo image
      g2.setColor(Color.black);
      g2.drawLine(0,0,w-1,0);
      g2.drawLine(w-1,0,w-1,h-1);
      g2.drawLine(w-1,h-1,0,h-1);
      g2.drawLine(0,h-1,0,0);
   }

   private void kresliTienASlnko(Graphics2D g2){
      double nSx,nSy,nSz; //vektor od stredu Zeme k Slnku
      double px,py,pz; //vektor od stredu Zeme k miestu na povrchu
      double phiS=-7.272e-5*t;
      while(phiS>Math.PI)phiS-=2*Math.PI;
      while(phiS<-Math.PI)phiS+=2*Math.PI;
      int di=3,dj=3;
      nSx=Math.sin(thetaS)*Math.cos(phiS);
      nSy=Math.sin(thetaS)*Math.sin(phiS);
      nSz=Math.cos(thetaS);
      //kreslenie tiena
      for(int j=0;j<h;j+=2*dj){
         for(int i=0; i<w-di; i+=di){
            px=Math.sin(sc.jToy(j))*Math.cos(sc.iTox(i));
            py=Math.sin(sc.jToy(j))*Math.sin(sc.iTox(i));
            pz=Math.cos(sc.jToy(j));
            if(nSx*px+nSy*py+nSz*pz<0) g2.drawLine(i,j,i,j);
            px=Math.sin(sc.jToy(j+dj))*Math.cos(sc.iTox(i+di/2));
            py=Math.sin(sc.jToy(j+dj))*Math.sin(sc.iTox(i+di/2));
            pz=Math.cos(sc.jToy(j+dj));
            if(nSx*px+nSy*py+nSz*pz<0) g2.drawLine(i+di/2,j+dj,i+di/2,j+dj);
         }
      }
      //kreslenie Slnka
      g2.setColor(Color.yellow);
      g2.fillOval(sc.xToi(phiS)-3,sc.yToj(thetaS)-3,6,6);
      g2.setColor(Color.black);
      g2.drawOval(sc.xToi(phiS)-3,sc.yToj(thetaS)-3,6,6);
   }

   private void kresliOblastViditelnosti(Graphics2D g2){
      double thetaP,alphaP;//suradnice bodu na obvode kruznice viditelnosti
      double thetaPStare, alphaPStare;
      double rho=Math.acos(Rz/r);
      g2.setColor(Color.red);
      double beta=0, dBeta=Math.PI/90.0;
      thetaPStare=Math.acos(Math.cos(theta)*Math.cos(rho)+Math.sin(theta)*Math.sin(rho)*Math.cos(beta));
      alphaPStare=Math.asin(Math.sin(beta)*Math.sin(rho)/Math.sin(thetaPStare));
      if(Math.cos(rho)-Math.cos(thetaPStare)*Math.cos(theta)<0) alphaPStare=Math.PI-alphaPStare;

      while(beta<Math.PI){
         beta+=dBeta;
         thetaP=Math.acos(Math.cos(theta)*Math.cos(rho)+Math.sin(theta)*Math.sin(rho)*Math.cos(beta));
         alphaP=Math.asin(Math.sin(beta)*Math.sin(rho)/Math.sin(thetaP));
         if(Math.cos(rho)-Math.cos(thetaP)*Math.cos(theta)<0) alphaP=Math.PI-alphaP;
         if(Math.abs(adjust(phi-alphaPStare)-adjust(phi-alphaP))<3.14)
            g2.drawLine(sc.xToi(adjust(phi-alphaPStare)),sc.yToj(thetaPStare),sc.xToi(adjust(phi-alphaP)),sc.yToj(thetaP));
         if(Math.abs(adjust(phi+alphaPStare)-adjust(phi+alphaP))<3.14)
            g2.drawLine(sc.xToi(adjust(phi+alphaPStare)),sc.yToj(thetaPStare),sc.xToi(adjust(phi+alphaP)),sc.yToj(thetaP));
         thetaPStare=thetaP;
         alphaPStare=alphaP;
      }
   }

   public double adjust(double phi){
      //najprv upravi phi vzhladom na rotaciu Zeme a potom
      //ho upravi tak, aby lezal v intervale od -PI do PI
      double phiA=phi-7.272e-5*t;
      while(phiA>Math.PI) phiA-=2*Math.PI;
      while(phiA<-Math.PI) phiA+=2*Math.PI;
      return phiA;
   }

   public void kresliSpodok(Graphics2D g2){
      //vykresli do pozadia mapu Zeme
      g2.drawImage(imMap, new AffineTransform(), null);
      //kreslenie suradnicovej siete
      g2.setColor(new Color(100,100,100));
      for(int i=-6; i<6; i++){
         g2.drawLine(sc.xToi(i*30/180.0*Math.PI),0,sc.xToi(i*30/180.0*Math.PI),h-1);
      }
      for(int j=1; j<6; j++){
         g2.drawLine(0,sc.yToj(j*30/180.0*Math.PI),w-1,sc.yToj(j*30/180.0*Math.PI));
      }
   }

   public void run(){
      while ((!koniec)&&(!dopad)){
         simulationStep();
         try {
            vlakno.sleep(50);
         }
         catch(InterruptedException exception){
            System.err.println("Interrupted...");
         }
      }
   }

   public void simulationStep(){
      double phiStareAdjusted, phiAdjusted;
      phiStare=ode.getPhi();
      phiStareAdjusted=adjust(phiStare);
      thetaStare=ode.getTheta();
      t+=odeAdaptiveSolver.step();
      //...vykonava sa krok Runge Kutteho metody
      phi=ode.getPhi();
      phiAdjusted=adjust(phi);
      theta=ode.getTheta();
      r=ode.getR();

      lblTime.setText("t = ".concat(MyUtils.roundOff(t,0)).concat(" s"));
      lblH.setText("h = ".concat(MyUtils.roundOff((r-Rz)/1000.0,1)).concat(" km"));
      lblSirka.setText("z..: ".concat(MyUtils.roundOff(toSirka(theta),1)).concat(" "));
      lblDlzka.setText("z.d.: ".concat(MyUtils.roundOff(toDlzka(phi),1)).concat(" "));

      if(r<Rz){//osetrenie dopadu na zem
         koniec=true;
         dopad=true;
         btnStartStop.setText("tart");
         btnStartStop.setIcon(iconPlay);
         btnDopredu.setEnabled(true);
         btnDozadu.setEnabled(true);
         btnNastav.setEnabled(true);
         btnReset.setEnabled(true);
         btnStav.setEnabled(true);
      }

      //kreslenie STOPY druzice
      if(Math.abs(phiStareAdjusted-phiAdjusted)<1.57){
         g2.setColor(Color.black);
         g2.drawLine(sc.xToi(phiStareAdjusted),sc.yToj(thetaStare),sc.xToi(phiAdjusted),sc.yToj(theta));
         platno.repaint();
      }
      if(dopad)JOptionPane.showMessageDialog(this,"Je mi to, ale druica spadla na Zem...","Upozornenie",JOptionPane.WARNING_MESSAGE);
   }

   private double toSirka(double theta){
      //vstup uhol theta v radianoch
      //vrzti zem. sirku v stupnoch
      double th=theta/Math.PI*180.0;
      return 90-th;
   }

   private double toDlzka(double phi){
     //vstup uhol phi v radianoch
     //vystup zem. dlzka v stupnoch
     double ph=phi/Math.PI*180.0;
     while(ph>180)ph=ph-360;
     while(ph<-180)ph=ph+360;
     return ph;
   }

   public void mousePressed(MouseEvent e){
   }

   public void mouseDragged(MouseEvent e){
   }

   public void mouseReleased(MouseEvent e){
   }

   public void mouseEntered(MouseEvent e){
      cursorOver=true;
      platno.repaint();
   }

   public void mouseExited(MouseEvent e){
      cursorOver=false;
      platno.repaint();
   }

   public void mouseClicked(MouseEvent e){
   }

   public void mouseMoved(MouseEvent e){
      ci=e.getX();
      cj=e.getY();
      platno.repaint();
   }

}

class TextualInputWindow extends JDialog implements ActionListener {
   JOrbitLab applet;
   JLabel lblH, lblLatitude, lblLongitude, lblVR, lblVNorth, lblVEast, lblDT;
   JTextField tfH, tfLatitude, tfLongitude, tfVR, tfVNorth, tfVEast, tfDT;
   JPanel pnlCenter, pnlButtons;
   JButton btnOK, btnCancel;
   String sH,sTheta,sPhi,sVR,sVTheta,sVPhi,sDT;

   //constructor
   public TextualInputWindow(JOrbitLab applet){
      setModal(true);
      setTitle("Nastavenia simulcie");
      setSize(200,340);
      setResizable(false);

      this.applet=applet;

      Container cp=getContentPane();
      btnOK=new JButton("OK");
      btnCancel=new JButton("Cancel");
      lblH = new JLabel("h / m:");
      lblLatitude = new JLabel("zemepisn rka / stupne:");
      lblLongitude = new JLabel("zemepisn dka / stupne:");
      lblVR = new JLabel("rchlos nahor / m/s:");
      lblVNorth = new JLabel("rchlos na sever / m/s:");
      lblVEast = new JLabel("rchlos na vchod / m/s:");
      lblDT = new JLabel("asov krok / s:");
      tfH= new JTextField(applet.parH);
      tfLatitude= new JTextField(applet.parFi);
      tfLongitude= new JTextField(applet.parLam);
      tfVR= new JTextField(applet.parVR);
      tfVNorth= new JTextField(applet.parVNorth);
      tfVEast= new JTextField(applet.parVEast);
      tfDT = new JTextField(applet.parDT);

      pnlCenter=new JPanel();
      pnlCenter.setLayout(new BoxLayout(pnlCenter,BoxLayout.Y_AXIS));
      pnlCenter.add(lblH);
      pnlCenter.add(tfH);
      pnlCenter.add(lblLatitude);
      pnlCenter.add(tfLatitude);
      pnlCenter.add(lblLongitude);
      pnlCenter.add(tfLongitude);
      pnlCenter.add(lblVR);
      pnlCenter.add(tfVR);
      pnlCenter.add(lblVNorth);
      pnlCenter.add(tfVNorth);
      pnlCenter.add(lblVEast);
      pnlCenter.add(tfVEast);
      pnlCenter.add(lblDT);
      pnlCenter.add(tfDT);

      pnlCenter.setAlignmentX(Component.LEFT_ALIGNMENT);
      pnlCenter.setPreferredSize(new Dimension(150,350));

      pnlCenter.add(Box.createVerticalGlue());
      pnlCenter.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));

      pnlButtons = new JPanel();
      pnlButtons.setLayout(new GridLayout(1,2));
      pnlButtons.add(btnOK);
      pnlButtons.add(btnCancel);
      pnlButtons.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));

      cp.add(pnlButtons,BorderLayout.SOUTH);
      cp.add(pnlCenter,BorderLayout.CENTER);

      btnOK.addActionListener(this);
      btnCancel.addActionListener(this);
      //nacitanie starych hodnot
      sH=tfH.getText();
      sTheta=tfLatitude.getText();
      sPhi=tfLongitude.getText();
      sVR=tfVR.getText();
      sVTheta=tfVNorth.getText();
      sVPhi=tfVEast.getText();
      sDT=tfDT.getText();
   }


   public void setLocation(){
      //Sets the location of Textual Input Window so that it is
      //centered at the applet...
      Point cpP=applet.getContentPane().getLocationOnScreen();
      int ic=Math.round((float)cpP.getX());
      int jc=Math.round((float)cpP.getY());
      setLocation(new Point(ic+80,jc+50));
   }

   public void actionPerformed(ActionEvent e){
      double r,theta,phi,vR,vTheta,vPhi,dt;//uhly v stupnoch!!!
      boolean vsetkoOK;
      if(e.getActionCommand().equals("OK")){
         //after pressing OK button...
         vsetkoOK=true;
         try {
            r=Double.valueOf(tfH.getText()).doubleValue()+6378e3;
            sH=tfH.getText();
            applet.rInitial=r;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfH.setText(sH);
            r=Double.valueOf(sH).doubleValue()+6378e3;
            vsetkoOK=false;
         }
         try {
            theta=90-Double.valueOf(tfLatitude.getText()).doubleValue();
            sTheta=tfLatitude.getText();
            applet.thetaInitial=theta;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfLatitude.setText(sTheta);
            theta=90-Double.valueOf(sTheta).doubleValue();
            vsetkoOK=false;
         }
         try {
            phi=Double.valueOf(tfLongitude.getText()).doubleValue();
            if(phi<0)phi=phi+360;
            sPhi=tfLongitude.getText();
            applet.phiInitial=phi;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfLongitude.setText(sPhi);
            phi=Double.valueOf(sPhi).doubleValue();
            if(phi<0)phi=phi+360;
            vsetkoOK=false;
         }
         try {
            vR=Double.valueOf(tfVR.getText()).doubleValue();
            sVR=tfVR.getText();
            applet.vRInitial=vR;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfVR.setText(sVR);
            vR=Double.valueOf(sVR).doubleValue();
            vsetkoOK=false;
         }
         try {
            vTheta=-Double.valueOf(tfVNorth.getText()).doubleValue();
            sVTheta=tfVNorth.getText();
            applet.vThetaInitial=vTheta;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfVNorth.setText(sVTheta);
            vTheta=-Double.valueOf(sVTheta).doubleValue();
            vsetkoOK=false;
         }
         try {
            vPhi=Double.valueOf(tfVEast.getText()).doubleValue();
            sVPhi=tfVEast.getText();
            applet.vPhiInitial=vPhi;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfVEast.setText(sVPhi);
            vPhi=Double.valueOf(sVPhi).doubleValue();
            vsetkoOK=false;
         }
         try {
            dt=Double.valueOf(tfDT.getText()).doubleValue();
            sDT=tfDT.getText();
            applet.dt=dt;
            applet.dtInitial=dt;
         }
         catch(NumberFormatException ex){
            JOptionPane.showMessageDialog(this,"Nesprvny formt vstupnej hodnoty!\nSkste to znovu.","Chyba vstupu",JOptionPane.ERROR_MESSAGE);
            tfDT.setText(sDT);
            dt=Double.valueOf(sDT).doubleValue();
            vsetkoOK=false;
         }
         //este overime, ci theta je v intervale od 0 do 180 a ci r je kladne
         while(theta>180)theta-=360;
         while(theta<-180)theta+=360;
         //tym sme dostali theta do intervalu (-180,180)
         if(theta<0){
            theta=-theta;
            phi=phi+180;
            vsetkoOK=false;
         }
         while(phi>360){
            phi-=360;
            vsetkoOK=false;
         }
         while(phi<0){
            phi+=360;
            vsetkoOK=false;
         }
         if(!vsetkoOK){
            double phiPom=phi;
            if(phiPom>180)phiPom=phi-360;
            tfLongitude.setText(MyUtils.roundOff(phiPom,4));
            tfLatitude.setText(MyUtils.roundOff(90-theta,4));
         }
         //tym sme phi dostali do intervalu (0,360)
         if(r-6378e3<0){
            r=6378e3;
            tfH.setText(MyUtils.roundOff(0,0));
            vsetkoOK=false;
         }

         if(vsetkoOK){
            applet.nastavSimulaciu(r,theta,phi,vR,vTheta,vPhi,0,dt);
            hide();
            applet.repaint();
         }
      }
      else {
         //after pressing Cancel button...
         tfH.setText(sH);
         tfLatitude.setText(sTheta);
         tfLongitude.setText(sPhi);
         tfVR.setText(sVR);
         tfVNorth.setText(sVTheta);
         tfVEast.setText(sVPhi);

         hide();
         applet.repaint();
      }
   }

}


class StepForwardHandler implements ActionListener {
   JOrbitLab applet;

   public StepForwardHandler(JOrbitLab applet){
      this.applet=applet;
   }

   public void actionPerformed(ActionEvent e){
      applet.simulationStep();
   }
}

class StepBackHandler implements ActionListener {
   JOrbitLab applet;

   public StepBackHandler(JOrbitLab applet){
      this.applet=applet;
   }

   public void actionPerformed(ActionEvent e){
      applet.odeAdaptiveSolver.setStepSize(-applet.dt);
      applet.simulationStep();
      applet.odeAdaptiveSolver.setStepSize(applet.dt);
   }
}

class Platno extends JPanel{
   BufferedImage image;
   JOrbitLab applet;

   //konstruktor
   public Platno(BufferedImage image, JOrbitLab applet){
      this.image=image;
      this.applet=applet;
   }

   public void paintComponent(Graphics g){
      Graphics2D g2=(Graphics2D)g;
      g2.drawImage(image,null,0,0);
      applet.kresliVrch(g2);
   }
}


